"""All prompts used for fact-checking subtasks prompting."""

CLAIM_EXTRACTION_PROMPT = {
    "system": "براہ کرم وہ دعویٰ فراہم کریں جس کی آپ حقیقت جانچنا چاہتے ہیں۔",
    "user": """آپ کو ایک ایسا متن دیا گیا ہے جس میں علم کے دعوے شامل ہیں۔ دعویٰ ایک بیان ہے جو کچھ سچ یا جھوٹ ہونے کا دعویٰ کرتا ہے، جس کی تصدیق انسانوں سے کی جا سکتی ہے۔ آپ کا کام یہ ہے کہ آپ دیے گئے متن میں سے ہر دعوے کو درست طریقے سے شناخت اور نکالیں۔ پھر، کسی بھی کورفرنس (ضمیر یا دوسرے حوالہ دینے والے اظہار) کو دعوے کی وضاحت کے لیے حل کریں۔ ہر دعویٰ مختصر (15 الفاظ سے کم) اور خود مختار ہونا چاہیے۔
متن اردو میں دیا گیا ہے اور دعوے اردو میں نکالے جانے چاہئیں۔
آپ کا جواب صرف نیچے دیے گئے فارمیٹ میں ہونا چاہیے۔ اس کے علاوہ کوئی اور اضافی نوٹس یا وضاحت شامل نہ کریں۔
[جواب کا فارمیٹ]:
[
{{
"claim": "یقین دہانی کرائیں کہ دعویٰ 15 الفاظ سے کم ہو اور مکمل خیال فراہم کرے۔ کورفرنس کو دعوے کی وضاحت کے لیے حل کریں",
}},
…
]

یہاں دو مثالیں دی گئی ہیں:
[text]: کرکٹ میچ میں شعیب ملک نے 50 رنز بنائے۔ وہ پاکستان کے بہترین کھلاڑی ہیں۔ شعیب ملک کو اگلے میچ میں شامل کیا جائے گا۔
[response]: [{{"claim": "شعیب ملک نے 50 رنز بنائے"}}, {{"claim": "شعیب ملک پاکستان کے بہترین کھلاڑی ہیں"}}, {{"claim": "شعیب ملک کو اگلے میچ میں شامل کیا جائے گا"}}]

[text]: لاہور میں موسم خوشگوار رہا۔ لوگ پارکوں میں چلنے پھرنے گئے۔ حکام نے کہا کہ کل بارش ہو سکتی ہے۔
[response]: [{{"claim": "لاہور میں موسم خوشگوار رہا"}}, {{"claim": "لوگ پارکوں میں چلنے پھرنے گئے"}}, {{"claim": "حکام نے کہا کہ کل بارش ہو سکتی ہے"}}]

اب یہ مکمل کریں، صرف جواب کی شکل میں، کوئی اور الفاظ نہیں:
[text]: {input}
[response]:
""",
}

QUERY_GENERATION_PROMPT = {
    "system": "آپ ایک سوالات بنانے والا ہیں جو دیے گئے دعوے کو تصدیق کرنے کے لیے موثر اور جامع تلاش کے انجن کے سوالات تیار کرتا ہے۔ آپ صرف پائیتھون کی فہرست کی شکل میں جواب دیں گے (کسی اور الفاظ میں نہیں!)",
    "user": """آپ ایک سوالات بنانے والے ہیں جو صارفین کو دیے گئے دعوے کو تلاش کے انجن کے ذریعے تصدیق کرنے میں مدد کرتے ہیں۔ آپ کا بنیادی کام دو موثر اور شک انگیز تلاش کے انجن کے سوالات تیار کرنا ہے۔ یہ سوالات صارفین کو دیے گئے دعوے کی حقیقت کو تنقیدی طور پر جانچنے میں مدد فراہم کریں گے۔
سوالات اردو میں ہونے چاہئیں اور سوالات اردو میں بنائے جائیں۔
آپ کو صرف نیچے دیے گئے فارمیٹ میں جواب دینا ہوگا (پائیتھون کی فہرست میں سوالات۔ براہ کرم اس فارمیٹ کی سختی سے پیروی کریں۔ کچھ اور واپس نہ کریں۔ اپنا جواب '[' سے شروع کریں۔
[جواب کا فارمیٹ]: ['سوال1', 'سوال2']

یہاں تین مثالیں ہیں:
دعویٰ: ٹویٹر کے سی ای او بل گیٹس ہیں۔
جواب: ["ٹویٹر کے سی ای او کون ہیں؟", "سی ای او ٹویٹر"]

دعویٰ: مائیکل فیلپس تمام اوقات کے سب سے زیادہ سجاوٹی اولمپین ہیں۔
جواب: ["تمام اوقات کے سب سے زیادہ سجاوٹی اولمپین کون ہیں؟", "مائیکل فیلپس"]

دعویٰ: چیٹ جی پی ٹی کو گوگل نے بنایا ہے۔
جواب: ["چیٹ جی پی ٹی کو کس نے بنایا؟", "چیٹ جی پی ٹی"]

اب یہ مکمل کریں، صرف جواب کی شکل میں، کوئی اور الفاظ نہیں:
دعویٰ: {input}
جواب:
""",
}

VERIFICATION_PROMPT = {
    "system": "آپ ایک شاندار معاون ہیں۔",
    "user": """آپ کو ایک ٹکڑا دیا گیا ہے۔ آپ کا کام یہ ہے کہ آپ یہ شناخت کریں کہ آیا دیے گئے متن میں کوئی حقیقت کی غلطیاں ہیں۔
جب آپ دیے گئے متن کی حقیقت کو پرکھ رہے ہوں، تو آپ ضرورت کے مطابق فراہم کردہ شواہد کا حوالہ دے سکتے ہیں۔ فراہم کردہ شواہد مددگار ہو سکتے ہیں۔ بعض شواہد ایک دوسرے سے متضاد ہو سکتے ہیں۔ آپ کو شواہد کو احتیاط سے استعمال کرنا چاہیے جب آپ دیے گئے متن کی حقیقت کا اندازہ لگائیں۔
جواب ایک ڈکشنری ہونی چاہیے جس میں تین کلیدیں ہوں - "reasoning" (وجہ)، "factuality" (حقیقت) اور "error" (غلطی) اور "correction" (تصحیح)، جو بالترتیب آپ کی وجہ، یہ کہ آیا دیے گئے متن میں کوئی حقیقتی غلطی ہے یا نہیں (Boolean - True یا False)، اور غلطی کی وضاحت، اور تصحیح فراہم کریں۔
وجہ، غلطی اور تصحیح اردو میں ہونی چاہیے۔
یہ ہے دی گئی عبارت
[text]: {claim}
یہ ہیں فراہم کردہ شواہد
[evidences]: {evidence}
آپ کو صرف نیچے دیے گئے فارمیٹ میں جواب دینا چاہیے۔ کچھ اور واپس نہ کریں۔ اپنے جواب کا آغاز '{{' سے کریں۔
[جواب کا فارمیٹ]:
{{
"reasoning": "کیوں دی گئی عبارت حقیقت پر مبنی ہے یا نہیں؟ جب آپ یہ کہتے ہیں کہ کوئی چیز حقیقت پر مبنی نہیں ہے، تو آپ کو اپنے فیصلے کی حمایت کرنے کے لیے متعدد شواہد فراہم کرنے ہوں گے۔",
"error": "اگر عبارت حقیقت پر مبنی ہے تو 'None'، ورنہ غلطی کی وضاحت کریں۔",
"correction": "اگر کوئی غلطی ہو تو تصحیح شدہ عبارت فراہم کریں۔",
"factuality": True اگر دی گئی عبارت حقیقت پر مبنی ہے، ورنہ False۔
}}
""",
}

URDU_TO_ENGLISH_TRANSLATION_PROMPT = {
    "system": "You are a helpful assistant.",
    "user": """You are given a piece of text in Urdu. Your task is to translate it into English. The translation should be accurate and maintain the original meaning of the text. Please ensure that the translation is grammatically correct and coherent in English.
DO NOT RESPOND WITH ANYTHING ELSE. ADDING ANY OTHER EXTRA NOTES THAT VIOLATE THE RESPONSE FORMAT IS BANNED. 

{input}
""",
}

ENGLISH_TO_URDU_TRANSLATION_PROMPT = {
    "system": "You are a helpful assistant.",
    "user": """You are given a piece of text in English. Your task is to translate it into Urdu. The translation should be accurate and maintain the original meaning of the text. Please ensure that the translation is grammatically correct and coherent in Urdu.
DO NOT RESPOND WITH ANYTHING ELSE. ADDING ANY OTHER EXTRA NOTES THAT VIOLATE THE RESPONSE FORMAT IS BANNED.

{input}
""",
}
